﻿function Game(canvasId, settings) {
    this.entities = new EntityManager();
    this.camera = new CameraControl();
    this.assets = new Assets();
    this.blood = new Blood();
    this.dots = [];
    this.map = new Map();
    this.backgroundMusic = null;
    this.win = false;

    this.resizeTheCanvas = function (w, h) {
        w = Math.floor(w);
        h = Math.floor(h);

        this.canvas.width = w;
        this.canvas.height = h;
        this.canvas.style.width = w + "px";
        this.canvas.style.height = h + "px";

        this.camera.resize(w, h);
    }

    this.draw = function (ctx, interpOffset) {
        ctx.save();
        this.camera.resize(ctx.canvas.width, ctx.canvas.height);
        this.camera.ensureVisible(this.player.renderX, this.player.renderY);
        this.map.render(ctx, this.camera, this.assets);
        ctx.restore();

        ctx.save();
        this.blood.render(ctx, this.camera);
        ctx.restore();

        var stage = this.entities;
        var i = 0;
        var max = stage.children.length;
        var now = (new Date()).getTime();
        for (i = 0; i < max; ++i) {
            drawSprite(stage.children[i], this.camera);
        }

        function drawSprite(sprite, camera) {
            if (sprite.animations) {
                sprite.animations.update(now);
            }
            var j;
            ctx.save();
            //Interpolation
            if (sprite.previousX !== undefined) {
                sprite.renderX = (sprite.x - sprite.previousX) * interpOffset + sprite.previousX;
            } else {
                sprite.renderX = sprite.x;
            }
            if (sprite.previousY !== undefined) {
                sprite.renderY = (sprite.y - sprite.previousY) * interpOffset + sprite.previousY;
            } else {
                sprite.renderY = sprite.y;
            }

            //camera.centerOn(sprite.renderX, sprite.renderY);

            //Draw the sprite at its interpolated position
            var xlateX;
            var xlateY;
            xlateX = (sprite.renderX - camera.x) + (sprite.width * sprite.pivotX);
            xlateY = (sprite.renderY - camera.y) + (sprite.height * sprite.pivotY);
            ctx.translate(xlateX, xlateY);

            // Rotation, alpha and scale
            ctx.rotate(sprite.rotation);
            ctx.globalAlpha = sprite.alpha * sprite.parent.alpha;
            ctx.scale(sprite.scaleX, sprite.scaleY);
            if (sprite.blendMode) ctx.globalCompositeOperation = sprite.blendMode;
            sprite.render(ctx);

            if (sprite.children && sprite.children.length > 0) {
                //Reset the context back to the parent sprite's top left corner,
                //relative to the pivot point
                ctx.translate(-sprite.width * sprite.pivotX, -sprite.height * sprite.pivotY);
                //Loop through the parent sprite's children
                for (j = 0; j < sprite.children.length; ++j) {
                    drawSprite(sprite.children[j], camera);
                }
            }
            ctx.restore();
        }

        ctx.save();
        ctx.font = "20px helvetica, arial, sans-serif";
        ctx.fillStyle = "#000";
        ctx.fillText("Shifty Shape Challenge", 10, 50);
        ctx.font = "18px helvetica, arial, sans-serif";
        ctx.fillText("Left Click to fly left", 10, 80);
        ctx.fillText("Right Click to fly right", 10, 100);
        ctx.fillText("Avoid the shifting shapes", 10, 120);

        if (this.won) {
            ctx.font = "32px helvetica, arial, sans-serif";
            ctx.fillStyle = "red";
            ctx.fillText("YOU WON!", this.canvas.width / 2 - 32, this.canvas.height / 2);
        }

        ctx.restore();
    }

    this.testWin = function(p) {
        var a = Math.abs(p.x - this.map.winPosX);
        var b = Math.abs(p.y - this.map.winPosY);

        var c = Math.sqrt(a * a + b * b);
        return c < 64;
    }

    this.update = function (elapsed, stage) {

        //if (this.controller.leftButton()) {
            //this.dots.push(this.controller.mousePos);
        //    this.blood.add(300, 200);
        //}
        this.blood.update(elapsed);

        var i;
        for (i = 0; i < stage.children.length; ++i) {
            stage.children[i].capturePreviousPosition();
        }

        if (!this.player.dead) {
            if (this.testWin(this.player)) {
                this.won = true;
            }

            if (this.player.x)
            for (i = 0; i < stage.children.length; ++i) {
                if (stage.children[i].testCollision && stage.children[i].testCollision(this.player)) {
                    this.blood.add(this.player.x + this.player.width / 2, this.player.y + this.player.height / 2);
                    this.player.die();
                    this.assets.playSound("blood");
                    break;
                }
            }
        } else {
            if ((new Date()).getTime() - this.player.deathTime > 3000) {
                this.player.dead = false;
                this.player.x = this.map.spawnX;
                this.player.y = this.map.spawnY;
            }
        }

        for (i = 0; i < stage.children.length; ++i) {
            stage.children[i].update(elapsed, this.map);
        }
        this.controller.clearButtons();

        for (i = 0; i < stage.children.length; ++i) {
            if (stage.children[i].sound) {
                this.assets.playSound(stage.children[i].sound);
                stage.children[i].sound = '';
            }
            if (stage.children[i].effect == 'blood') {
                this.blood.add(stage.children[i].x, stage.children[i].y);
            }
        }

    }

    this.gameLoop = function (timeStamp) {
        "use strict";

        var that = this;
        if (window.requestAnimationFrame) {
            window.requestAnimationFrame(function () { that.gameLoop((new Date()).getTime()); });
        } else {
            setTimeout(function () { that.gameLoop((new Date()).getTime()); }, Math.floor(1000 / 60));
        }

        if (!timeStamp) {
            timeStamp = 0;
        }
        var elapsed = timeStamp - this.previous;

        // Correct large gaps
        if (elapsed > 1000) {
            elapsed = this.frameDuration;
        }

        this.lag += elapsed;

        // Update the frame if lag is larger than the frame duration
        while (this.lag >= this.frameDuration) {
            //capturePreviousPositions(stage);
            this.update(elapsed, this.entities);
            this.lag -= this.frameDuration;
        }

        var interpOffset = this.lag / this.frameDuration;

        this.draw(this.ctx, interpOffset);
        this.previous = timeStamp;
    }
    settings = settings || { width: 1066, height: 600 };
    this.canvas = document.getElementById(canvasId);

    this.canvas.style.msTouchAction = 'none';
    this.canvas.style.touchAction = 'none';

    this.ctx = this.canvas.getContext('2d');

    this.controller = new Controller(this.canvas);

    this.previous = 0;  // Previous time stamp
    this.fps = 30;      // Physics fps
    this.frameDuration = 1000 / this.fps,
    this.lag = 0;

    this.resizeTheCanvas(settings.width, settings.height);

    // Queue images
    if (typeof WinJS !== "undefined") {
        this.assets.setBase("../../assets/images/");  // Windows base
    } else {
        this.assets.setBase("assets/images/");
    }
    //    this.assets.queue("player", "lightning.png"); //"player1.png"); //"Frame-1.png");
    //this.assets.queue("player", "birdr2.png"); //"Frame-1.png");

    this.assets.queue("birdl1", "birdl1.png");
    this.assets.queue("birdl2", "birdl2.png");
    this.assets.queue("birdl3", "birdl3.png");
    this.assets.queue("birdl4", "birdl4.png");
    this.assets.queue("birdr1", "birdr1.png");
    this.assets.queue("birdr2", "birdr2.png");
    this.assets.queue("birdr3", "birdr3.png");
    this.assets.queue("birdr4", "birdr4.png");

    this.assets.queue("map1", "map1.png");
    this.assets.queue("sky", "tile.sky.png");
    this.assets.queue("grass", "tile.wall.png");//wall.png");
    this.assets.queue("invalid", "tile.invalid.png");

    // Queue sounds
    if (typeof WinJS !== "undefined") {
        this.assets.setBase("../../assets/sounds/");  // Windows Store base
    } else {
        this.assets.setBase("assets/sounds/");
    }
    this.assets.queue("music1", "240376__edtijo__happy-8bit-pixel-adenture.mp3", 1);
    this.assets.queue("blood", "266890__christopherderp__greasy-meat-slap.mp3");
    this.assets.queue("bounce", "51461__andre-rocha-nascimento__basket-ball-02-bounce.mp3");

    // Load assets and start game loop
    var that = this;
    this.assets.load(function () {
        that.backgroundMusic = that.assets.get("music1");
        //that.backgroundMusic.volume = 0.1;
        that.backgroundMusic.loop = true;
        that.backgroundMusic.play();

        that.resizeTheCanvas(settings.width, settings.height);
        that.map.loadFromImage(that.assets.get("map1"));

        //that.assets.get("bounce").play();
        //that.assets.playSound("bounce");

        that.player = new Player(that.controller, that.assets, 96, 32 + 16, 32, 32);
        that.entities.addChild(that.player);
        that.map.createEntities(that.entities);
        that.player.x = that.map.spawnX;
        that.player.y = that.map.spawnY;

//        that.entities.addChild(new Shrinker(250, 250, 100, 24));
        //that.entities.addChild(new Sprite(that.assets.get("player"), 200, 200, 32, 32));
        var k = 0;
        for (k = 0; k < 0; ++k) {
            that.entities.addChild(new Sprite(that.assets.get("player"), Math.random() * that.canvas.width, Math.random() * that.canvas.height, 32, 32));
        }

        that.gameLoop((new Date()).getTime());
    });
}
