﻿function Assets() {
    var assets = [];
    var queue = [];
    var loadedSoFar = 0;
    var toLoad = 0;
    var currentBase = '';
    this.soundEnabled = true;


    var imageExtensions = ["png", "jpg", "jpeg", "gif"];
    var audioExtensions = ["mp3", "ogg", "wav", "webm"];

    var audioTypes = {
        mp3: { type: "audio/mpeg" },
        ogg: { type: "audio/ogg" },
        webm: { type: "audio/webm" },
        aac: { type: "audio/aac" },
        wav: { type: "audio/wav" },
    };

    this.testSound = function () {
        var iOS = /iPad|iPhone|iPod/.test(navigator.userAgent) && !window.MSStream;
        if (iOS) {
            return false;
        }

        var testNode = document.createElement("Audio");
        if (!testNode) {
            return false;
        } else if (typeof testNode.canPlayType === "undefined") {
            return false;
        } else if (!testNode.canPlayType("audio/mpeg")) {
            return false;
        } else {
            return true;
        }
    }
    this.soundEnabled = this.testSound();

    this.setBase = function (base) {
        currentBase = base;
    }

    this.playSound = function (key) {
        if (!this.soundEnabled || assets[key] === undefined) {
            return false;
        }

        var now = (new Date()).getTime();
        var assetInfo = this.getInfo(key);
        var diff;
        if (assetInfo.lastPlayed && (now - assetInfo.lastPlayed) < 125) {
            diff = now - assetInfo.lastPlayed;
            return;
        }
        assetInfo.lastPlayed = now;
        var theAudio;
        theAudio = this.get(key);
        if (theAudio) { theAudio.play(); }
        return;
    }

    this.getInfo = function (key) {
        if (assets[key] === undefined) {
            return false;
        }
        return assets[key];
    }

    this.get = function (key) {
        if (assets[key] === undefined) {
            return false;
        }
        var asset = assets[key];
        var ob = asset.objects[asset.nextTrack];
        asset.nextTrack = (asset.nextTrack + 1) % asset.objects.length;
        return ob;
    }

    this.queue = function (key, url, channels) {
        channels = channels || 4;
        queue.push({ key: key, url: currentBase + url, channels: channels});
    }

    this.load = function (callback) {
        var i = 0;
        var j = 0;
        var key;
        var ob;
        for (i = 0; i < queue.length; ++i) {
            var ext = queue[i].url.split(".").pop();

            key = queue[i].key;
            if (imageExtensions.indexOf(ext) !== -1) {
                assets[key] = { objects: [], nextTrack: 0, channels: 0 };
                ++toLoad;
                ob = new Image();
                ob.addEventListener("load", onLoaded, false);
                assets[key].objects[0] = ob;
                assets[key].objects[0].src = queue[i].url;
            } else if (audioExtensions.indexOf(ext) !== -1 && this.soundEnabled) {
                assets[key] = { objects: [], nextTrack: 0, channels: 0 };
                j = 0;
                toLoad += queue[i].channels;
                for (j = 0; j < queue[i].channels; ++j) {
                    ob = new Audio();
                    ob.addEventListener("canplaythrough", onLoaded, false);
                    assets[key].objects[j] = ob;
                    assets[key].objects[j].src = queue[i].url;
                }
            }
        }

        function onLoaded() {
            ++loadedSoFar;
            if (loadedSoFar === toLoad) {
                callback();
            }
        }
    }
}
