﻿var entityId = 0;

function Entity(x, y, w, h) {
    "use strict";

    ++entityId;
    this.id = entityId;

    this.x = x || 0;
    this.y = y || 0;
    this.width = w || 100;
    this.height = h || 100;

    this.dead = false;
    this.deathTime = 0;

    this.rotation = 0;
    this.scaleX = 1.0;
    this.scaleY = 1.0;
    this.alpha = 1.0;
    this.blendMode = null;  // "source-over" is the default

    this.previousX = x;
    this.previousY = y;

    this.accelerationX = 0;
    this.accelerationY = 0;

    this.gravity = 1000;
    this.friction = 0;

    this.vx = 0;
    this.vy = 0;

    this.pivotX = 0.5;
    this.pivotY = 0.5;

    this.renderX = x;
    this.renderY = y;

    this.children = [];

    this.sound = '';        // Name of sound to play after update
    this.effect = '';       // Name of particle effect to start after update
}

Entity.prototype.playSound = function (key) {
    this.sound = key;
}

Entity.prototype.die = function (key) {

    if (this.dead) {
        return;
    }
    this.dead = true;
    this.deathTime = (new Date()).getTime();
}

Entity.prototype.playEffect = function (key) {
    this.effect = key;
}

Entity.prototype.addChild = function (child) {
    this.children.push(child);
    child.parent = this;
},
    
Entity.prototype.removeChild = function (child) {
    this.children.splice(this.children.indexOf(child), 1);
}

Entity.prototype.capturePreviousPosition = function () {
    this.previousX = this.x;
    this.previousY = this.y;

    var i = 0;
    for (i = 0; i < this.children.length; ++i) {
        this.children[i].capturePreviousPosition();
    }
}

Entity.prototype.update = function (elapsed, map) {

    var m = elapsed / 1000;

    this.vx += this.accelerationX;
    this.vy += this.accelerationY;

    this.vy += this.gravity * m;
    this.vx -= Math.min(1.0, this.friction) * this.vx * m;
    this.vy -= Math.min(1.0, this.friction) * this.vy * m;

    this.accelerationX = 0;
    this.accelerationY = 0;

    var collision;
    this.x += this.vx * m;
    collision = map.testForCollision(this);
    if (collision.collision) {
        this.x += collision.h;
        this.vx = -this.vx;
        this.playSound('bounce');
    }

    this.y += this.vy * m;
    this.vy = Math.max(-1000, this.vy);
    this.vy = Math.min(1000, this.vy);
    collision = map.testForCollision(this);
    if (collision.collision) {
        this.y += collision.v; //&= ~32;
        this.vy = -this.vy;
        this.playSound('bounce');
    }

/*
    if (this.vy > 0 && this.y > 500) {
        this.vy = -this.vy;
    } else if (this.vy < 0 && this.y < 0) {
        this.vy = -this.vy;
    }*/
}

Entity.prototype.render = function (ctx) {
//        ctx.strokeStyle = this.strokeStyle;
//        ctx.lineWidth = this.lineWidth;

    ctx.fillStyle = "white";
    ctx.beginPath();
    ctx.rect(
        -this.width * this.pivotX,
        -this.height * this.pivotY,
        this.width,
        this.height
    );
    ctx.fill();
}
