﻿function Map() {
    this.map = [];
    this.w = 0;
    this.h = 0;
    this.lookup = [];
    this.status = '';
    this.spawnX = 0;
    this.spawnY = 0;
    //var h = this.map[0].toString(16);
    this.lookup[this.fudgify(parseInt('00FFFF', 16))] = "sky";
    this.lookup[this.fudgify(parseInt('FF00FF', 16))] = "grass";
    this.lookup[this.fudgify(parseInt('FFFF00', 16))] = "shiftery";
    this.lookup[this.fudgify(parseInt('00FF00', 16))] = "shifterg";
    this.lookup[this.fudgify(parseInt('0000FF', 16))] = "shifterb";
    this.lookup[this.fudgify(parseInt('FF0000', 16))] = "spawn";
    this.lookup[this.fudgify(parseInt('000000', 16))] = "winpos";
    this.clear = this.fudgify(parseInt('00FFFF', 16));
}

Map.prototype.testForCollisionPixelXY = function (x, y) {
    var tilePos;
    x = Math.floor(x / 32);
    y = Math.floor(y / 32);
    if (x < 0 || y < 0 || x > this.w || y > this.h) {
        return true;
    }

    tilePos = (y * this.w) + x;
    if (!this.checked) {
        this.checked = 0;
    }

    return this.map[tilePos] !== this.clear;
}

Map.prototype.testForCollision = function (ent) {
    var collision = { collision: false, h: 0, v: 0 };

    var testX, testY;
    testX = Math.floor(ent.x);
    testY = Math.floor(ent.y);
    if (this.testForCollisionPixelXY(testX, testY)) {
        collision.collision = true;
        collision.h = (Math.floor((testX + 32)/32) * 32) - testX; // Move right
        collision.v = (Math.floor((testY + 32) / 32) * 32) - testY; // Move down

//        collision.h = ((testX + 32) & ~31) - testX; // Move right
//        collision.v = ((testY + 32) & ~31) - testY; // Move down
        return collision;
    }

    testX = Math.floor(ent.x) + ent.width - 1;
    testY = Math.floor(ent.y);
    if (this.testForCollisionPixelXY(testX, testY)) {
        collision.collision = true;
        collision.h = ((testX & ~31) - 1) - testX;      // Move left
        collision.v = ((testY + 32) & ~31) - testY;     // Move down
        return collision;
    }

    // bottom left
    testX = Math.floor(ent.x);
    testY = Math.floor(ent.y) + ent.height - 1;
    if (this.testForCollisionPixelXY(testX, testY)) {
        collision.collision = true;
        collision.h = ((testX + 32) & ~31) - testX;     // Move right
        collision.v = ((testY & ~31) - 1) - testY;      // Move up
        return collision;
    }

    // bottom right
    testX = Math.floor(ent.x) + ent.width - 1;
    testY = Math.floor(ent.y) + ent.height - 1;
    if (this.testForCollisionPixelXY(testX, testY)) {
        collision.collision = true;
        collision.h = ((testX & ~31) - 1) - testX;      // Move left
        collision.v = ((testY & ~31) - 1) - testY;      // Move up
        return collision;
    }
    return false;
}

Map.prototype.fudgify = function (val) {

    var a = [];
    var i;
    for (i = 0; i < 3; ++i) {
        a[i] = val & 255;
        if (a[i] < 128) {
            a[i] = 0;
        } else {
            a[i] = 255;
        }
        val = val >> 8;
    }

    val = 0;
    for (i = 2; i >= 0; --i) {
        val = val << 8;
        val += a[i];
    }
    return val;
}

Map.prototype.getTile = function (x, y) {
    var tilePos;
    var tileVal;
    var tileName;
    x = Math.floor( x / 32);
    y = Math.floor( y / 32);
    if ( x < 0 || y < 0 || x >= this.w || y >= this.h) {
        return 'sky';
    }

    tilePos = (y * this.w) + x;
    tileVal = this.map[tilePos];
    tileVal = this.fudgify(tileVal);

    if (typeof this.lookup[tileVal] === 'string') {
        return this.lookup[tileVal];
    }
    return 'invalid';
}

Map.prototype.loadFromImage = function (img) {
    var imageData;
    var x;
    var y;
    var spos;
    var dpos;

    this.w = img.width;
    this.h = img.height;

    var c = document.createElement("canvas");
    var ctx = c.getContext('2d');

    ctx.save();
    ctx.imageSmoothingEnabled = false;
    
    //ctx.translate(.5, .5);

    c.width = c.style.width = this.w;
    c.height = c.style.height = this.h;

    ctx.drawImage(img, 0, 0);
    imageData = ctx.getImageData(0, 0, this.w, this.h);

    spos = 0;
    dpos = 0;
    i = 0;
    for (y = 0; y < imageData.height; ++y) {
        spos = y * imageData.width * 4;
        dpos = y * imageData.width;
        for (x = 0; x < imageData.width; ++x) {
            this.map[dpos] = this.fudgify(imageData.data[spos + 0] * 65536 + imageData.data[spos + 1] * 256 + imageData.data[spos + 2]); // Ignore alpha
            spos += 4;
            dpos += 1;
        }
    }
    ctx.restore();
}

Map.prototype.createEntities = function (entities) {
    var x;
    var y;
    var tileName;
    for (y = 0; y < this.h; ++y) {
        for (x = 0; x < this.w; ++x) {
            tileName = this.getTile(x * 32, y * 32);
            if (tileName == 'shiftery') {
                entities.addChild(new Shrinker(x * 32, y * 32, 64, 32, "yellow"));
                this.clearTile(x, y);
            } else if (tileName == 'shifterb') {
                entities.addChild(new Shrinker(x * 32, y * 32, 64, 32, "blue"));
                this.clearTile(x, y);
            } else if (tileName == 'shifterg') {
                entities.addChild(new Shrinker(x * 32, y * 32, 64, 32, "green"));
                this.clearTile(x, y);
            } else if (tileName == 'spawn') {
                    this.spawnX = x * 32;
                    this.spawnY = y * 32;
                    this.clearTile(x, y);
            } else if (tileName == 'winpos') {
                this.winPosX = x * 32;
                this.winPosY = y * 32;
                this.clearTile(x, y);
            }
            tileName = '';
        }
    }
}

Map.prototype.setTile = function (x, y, val) {
    this.map[y * this.w + x] = val;
}

Map.prototype.clearTile = function (x, y) {
    this.map[y * this.w + x] = this.clear;
}

Map.prototype.render = function (ctx, camera, assets) {
    var x, y;
    var tileName;
    var img;
    var renderX;
    var renderY;
    var maxX = (camera.x + camera.w + 32);
    var maxY = (camera.y + camera.h + 32);

    for (y = camera.y; y <= maxY; y += 32) {
        for (x = camera.x; x <= maxX; x += 32) {
            tileName = this.getTile(x, y);
            img = assets.get(tileName);
            if (img) {
                //ctx.drawImageimg, (0, 0, img.width, img.height, Math.floor(x / 32) * 32, Math.floor(y / 32) * 32, 32, 32);
                renderX = Math.floor(x / 32) * 32 - camera.x;
                renderY = Math.floor(y / 32) * 32 - camera.y;
                ctx.drawImage(img, 0, 0, 32, 32,
                renderX, renderY, 32, 32);
            }
        }
    }
}

